#
# generator.py
#

import problem
import data
import random
import re

print 'Loading word list.'
words = data.load('words.txt').read().split()
words = [word.lower() for word in words]
max_word = len(words) - 1
print 'Loaded %d words.' % (len(words))

random.seed()

vowels = ['a','e','i','o','u']

vowel_class = '[aeiou]'
not_vowel_class = '[^aeiou]'

repeaters = ['a','b','d','e','f','g','l','m','n','o','p','r','s','t']

weighted_list = list('aaaaaaaaabbccddddeeeeeeeeeeeeffggghhiiiiiiiiijkllllmmnnnnnnooooooooppqrrrrrrssssttttttuuuuvvwwxyyz')

max_wlist = len(weighted_list) - 1



rep_max = len(repeaters) - 1

def gen_char():
  rand_char = random.randint(97,122)
  return chr(rand_char)

def weighted_gen_char():
  rand_char = weighted_list[random.randint(0,max_wlist)]
  return rand_char

def gen_vowel():
  return vowels[random.randint(0,4)]
  
def gen_repeater():
  return repeaters[random.randint(0,rep_max)]

def gen_class(lower,upper):
  elem = '['
  for i in range(random.randint(lower,upper)):
    elem += weighted_gen_char()
  elem += ']'
  return elem

def gen_elem_easy():
  rand_sel = random.randint(0,99)
  elem = weighted_gen_char()
  if rand_sel < 80:
    return elem
  elif rand_sel < 90:
    return elem + '+'
  elif rand_sel < 100:
    return '.'

def gen_elem_medium():
  rand_sel = random.randint(0,99)
  if rand_sel < 90:
    return gen_elem_easy()
  elem = weighted_gen_char()
  if rand_sel < 92:
    return '.*'
  elif rand_sel < 96:
    return elem + '*'
  elif rand_sel < 100:
    return '[^' + elem + ']'

def gen_elem_hardish():
  rand_sel = random.randint(0,99)
  if rand_sel < 90:
    return gen_elem_medium()
  elif rand_sel < 100:
    return gen_class(2,3)
  
def gen_elem_hard():
  rand_sel = random.randint(0,99)
  if rand_sel < 60:
    return gen_elem_medium()
  else:
    return gen_elem_crazy()

def gen_elem_crazy():
  rand_sel = random.randint(0,99)
  if rand_sel < 20:
    return gen_vowel()
  if rand_sel < 40:
    return gen_char()
  elif rand_sel < 50:
    res = ''
    res += '[^'
    for n in range(random.randint(1,3)):
      res += gen_char()
    res += ']'
    return res
  elif rand_sel < 60:
    res = ''
    res += '['
    for n in range(random.randint(2,3)):
      res += gen_char()
    res += ']'
    return res
  elif rand_sel < 70:
    return gen_repeater() + '*'
  elif rand_sel < 80:
    return gen_repeater() + '+'
  elif rand_sel < 90:
    return vowel_class
  elif rand_sel < 100:
    return not_vowel_class

def anchor(regex, chance):
  rand_sel = random.randint(0,99)
  if rand_sel < chance:
    rand_sel = random.randint(0,1)
    if rand_sel == 0:
      return '\\b' + regex
    else:
      return regex + '\\b'
  return regex
    
def gen_regex(level):
  regex = ''
  if level == 1:
    regex += weighted_gen_char()
  elif level == 2: 
    regex += weighted_gen_char()
    regex += weighted_gen_char()
  elif level == 3: 
    for i in range(3):
      regex += gen_elem_easy()
  elif level == 4:
    for i in range(random.randint(3,4)):
      regex += gen_elem_easy()
    regex = anchor(regex, 10) 
  elif level == 5:
    for i in range(random.randint(3,4)):
      regex += gen_elem_medium()
    regex = anchor(regex, 20)
  elif level == 6:
    for i in range(random.randint(3,5)):
      regex += gen_elem_medium()
    regex = anchor(regex, 30)
  elif level == 7:
    for i in range(random.randint(3,5)):
      regex += gen_elem_hardish()
    regex = anchor(regex, 20)
  else:
    upper = level - 3 
    if upper > 10:
      upper = 10
    for i in range(random.randint(4,upper)):
      regex += gen_elem_hard()
    regex = anchor(regex, 20)
  return regex

def generate_problem(level):
  keep_trying = True
  while(keep_trying):
    regex = gen_regex(level)
    # print 'regex = ' + regex
    counter = 0
    match = 0
    notmatch = 0
    goodwords = []
    badwords = []
    num_words = 2 + level * 3
    if num_words > 18:
      num_words = 18
    max_word_length = level + 4
    while (counter < 10000) and ((match < num_words) or (notmatch < num_words)):
      counter += 1
      rand_word = words[random.randint(0,max_word)]
      if len(rand_word) > max_word_length:
        continue
      mo = re.search(regex, rand_word)
      if mo:
        match += 1
        if len(goodwords) < num_words:
          goodwords.append(rand_word)
      else:
        notmatch += 1
        if len(badwords) < num_words:
          badwords.append(rand_word)
    if counter < 10000:
      new_prob = problem.problem()
      new_prob.title = 'Level ' + str(level)
      new_prob.explanation = 'This is a level %d puzzle. ' % level
      new_prob.goodwords = goodwords
      new_prob.badwords = badwords
      new_prob.regex = regex
      keep_trying = False
      return new_prob

if __name__ == '__main__':
  for i in range(20):
    level = i + 1
    prob = generate_problem(level)
    print 'Level %d regex : %s' % (level, prob.regex)
